'use strict';

var solidJs = require('solid-js');

// src/index.ts
var SANITIZE_REGEX = /[^\w\s]/g;
var SPLIT_WORDS_REGEX = /\s+/g;
var NEVER_REGEX = /.^/;
function makeSearchRegex(search) {
  search = search.trim().replace(SANITIZE_REGEX, "");
  return search ? new RegExp(
    // join words `|` to match any of them
    search.replace(SPLIT_WORDS_REGEX, "|"),
    "gi"
  ) : NEVER_REGEX;
}
var disposeAll = (list) => {
  for (const item of list)
    item.dispose();
};
function createMarker(mapMatch, options) {
  const { cacheSize = 100 } = options ?? {}, toReuse = [], owner = solidJs.getOwner(), limitCache = () => toReuse.length > cacheSize && disposeAll(toReuse.splice(0, toReuse.length - cacheSize));
  solidJs.onCleanup(() => {
    disposeAll(toReuse);
    toReuse.length = 0;
  });
  return (string, regex) => {
    let match = regex.exec(string), lastIndex = 0, reuseIndex = 0;
    if (!match || !match[0])
      return [string];
    const parts = [], reusableList = [];
    do {
      const matchText = match[0];
      let reusable = toReuse[reuseIndex];
      if (reusable) {
        reuseIndex++;
        reusable.set(matchText);
      } else {
        solidJs.createRoot((dispose) => {
          const [text, set] = solidJs.createSignal(matchText);
          reusable = { el: mapMatch(text), set, dispose };
        }, owner);
      }
      lastIndex !== match.index && parts.push(string.slice(lastIndex, match.index));
      parts.push(reusable.el);
      reusableList.push(reusable);
      lastIndex = match.index + matchText.length;
    } while (match = regex.exec(string));
    toReuse.splice(0, reuseIndex);
    parts.push(string.slice(lastIndex));
    solidJs.onCleanup(() => {
      toReuse.push.apply(toReuse, reusableList);
      setTimeout(limitCache);
    });
    return parts;
  };
}

exports.createMarker = createMarker;
exports.makeSearchRegex = makeSearchRegex;
